/*      NTCONIO.C       Operating specific video & keyboard functions
 *                      for the Window NT operating system (console mode)
 *                      for MicroEMACS 3.11
 *                      (C)Copyright 1988 by Daniel M. Lawrence
 *                      Windows NT version by Walter Warniaha
 *
 * The routines in this file provide video and keyboard support using the
 * Windows NT console functions.
 *
 */

#define BUGGY_CODE 1
#define termdef 1                       /* don't define "term" external */

#undef  PASCAL
#undef  NEAR
#undef  HIBYTE

#include        "estruct.h"
#include        "eproto.h"
#include        "edef.h"
#include        "elang.h"

#if     WINNTCON

#define NROW    60              /* Screen size.                 */
#define NCOL    132             /* Edit if you want to.         */
#define MARGIN  8               /* size of minimim margin and   */
#define SCRSIZ  64              /* scroll size for extended lines */
#define NPAUSE  5               /* # times thru update to pause */

/* Forward references.          */

PASCAL NEAR ntmove();
PASCAL NEAR nteeol();
PASCAL NEAR nteeop();
PASCAL NEAR ntbeep();
PASCAL NEAR ntopen();
PASCAL NEAR ntclose();
PASCAL NEAR ntgetc();
PASCAL NEAR ntputc();
PASCAL NEAR ntflush();
PASCAL NEAR ntrev();
PASCAL NEAR ntkclose();
PASCAL NEAR ntkopen();
PASCAL NEAR ntcres();
PASCAL NEAR ntparm();
#if     COLOR
PASCAL NEAR ntfcol();
PASCAL NEAR ntbcol();
#endif
static WORD near ntAttribute(void);

/* Screen buffer to write to. */
static CHAR_INFO ciScreenBuffer[NROW * NCOL];
static int      cfcolor = 0;    /* current foreground color     */
static int      cbcolor = 15;   /* current background color     */
static int      ctrans[] =      /* ansi to ibm color translation table  */
        { 0, 4, 2, 6, 1, 5, 3, 7, 8, 12, 10, 14, 9, 13, 11, 15 };

static WORD ntrow = 0;         /* current cursor row   */
static WORD ntcol = 0;         /* current cursor col   */
static WORD ntMax = 0;
static WORD ntMin = (WORD)-1;
static WORD ntColMax = 0;
static WORD ntColMin = (WORD)-1;

int revflag = FALSE;                    /* are we currently in rev video? */

/* Windows NT Console screen buffer handle.  All I/O is done through this
 * handle.
 */
static HANDLE hInput, hOutput;
static char chConsoleTitle[256];    // Preserve the title of the console.
static long ConsoleMode, OldConsoleMode;

static WORD wNumKeyEvent = 0;
static INPUT_RECORD ir;
static WORD wKeyEvent;

/*
 * Standard terminal interface dispatch table.
 */
TERM    term    = {
        NROW-1,
        NROW-1,
        NCOL,
        NCOL,
        0, 0,
        MARGIN,
        SCRSIZ,
        NPAUSE,
        ntopen,
        ntclose,
        ntkopen,
        ntkclose,
        ntgetc,
        ntputc,
        ntflush,
        ntmove,
        nteeol,
        nteeop,
        nteeop,
        ntbeep,
        ntrev,
        ntcres
#if     COLOR
        , ntfcol,
        ntbcol
#endif
};


#if     COLOR
/*----------------------------------------------------------------------*/
/*      ntfcol()                                                        */
/* Set the current foreground color.                                    */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntfcol(
        int color)                      /* color to set */
{
    cfcolor = ctrans[color];
}

/*----------------------------------------------------------------------*/
/*      ntbcol()                                                        */
/* Set the current background color.                                    */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntbcol(
        int color)              /* color to set */
{
    cbcolor = ctrans[color];
}
#endif

static void near ntSetUpdateValues(void)
{
    if (ntrow < ntMin)
        ntMin = ntrow;
    if (ntrow > ntMax)
        ntMax = ntrow;
    if (ntMax == ntMin)
    {
        if (ntcol < ntColMin)
            ntColMin = ntcol;
        if (ntcol > ntColMax)
            ntColMax = ntcol;
    }
}

/*----------------------------------------------------------------------*/
/*      ntmove()                                                        */
/* Move the cursor.                                                     */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntmove(
        int row,
        int col)
{
    COORD dwCursorPosition;

    ntcol = dwCursorPosition.X = col;
    ntrow = dwCursorPosition.Y = row;
    SetConsoleCursorPosition(hOutput, dwCursorPosition);
}


/*----------------------------------------------------------------------*/
/*      ntflush()                                                       */
/* Update the physical video buffer from the logical video buffer.      */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntflush(void)
{
    SMALL_RECT srWriteRegion;
    COORD coordUpdateBegin, coordBufferSize;

    if (ntMin <= ntMax)
    {
#if BUGGY_CODE
        if (ntMin == ntMax)
        {
            /* Fri Feb 14 1992 WaltW - Same fuckin' line bud. */
            srWriteRegion.Right = ntColMax;
            srWriteRegion.Left = 0; //ntColMin;
        
            coordUpdateBegin.X = 0; //ntColMin;
            coordUpdateBegin.Y = ntMin;
        }
        else
#endif
        {
            srWriteRegion.Right = term.t_ncol - 1;
            srWriteRegion.Left = 0;
        
            coordUpdateBegin.X = 0;
            coordUpdateBegin.Y = ntMin;
        }    
        srWriteRegion.Bottom = ntMax;
        srWriteRegion.Top = ntMin;

        coordBufferSize.X = term.t_ncol;
        coordBufferSize.Y = term.t_nrow + 1;

        WriteConsoleOutput(hOutput, (PCHAR_INFO)ciScreenBuffer,
            coordBufferSize, coordUpdateBegin, &srWriteRegion);
        ntColMax = ntMax = 0;
        ntColMin = ntMin = (WORD) -1;
    }
    ntmove(ttrow, ttcol);
    return (TRUE);
}

static void near MouseEvent(void)
{
    return;
}

static void near WindowSizeEvent(void)
{
    term.t_nrow = ir.Event.WindowBufferSizeEvent.dwSize.Y - 1;
    term.t_ncol = ir.Event.WindowBufferSizeEvent.dwSize.X;
    ntflush();
    SetConsoleTitle("WindowSizeEvent");
}

static void near KeyboardEvent(void)
{
    if (ir.Event.KeyEvent.bKeyDown == FALSE)
        return;

    switch (ir.Event.KeyEvent.wVirtualScanCode)
    {
        case 17:            // Left Control Key
        case 88:            // Right Control Key
        case 18:            // Left Shift
        case 89:            // Right Shift
        case 25:            // Left Alt
        case 57:            // Right Alt
        case 20:            // Caps Lock
        case 118:           // Num Lock
        case 95:            // Scroll Lock
        case 98:            // Pause
            return;
    }

    /* Wed Jan 22 1992 WaltW - Now we need to convert the NT key event to
     * EMACS.
     */
    switch (ir.Event.KeyEvent.wVirtualKeyCode)
    {
        case 121:       // F10
            ir.Event.KeyEvent.wVirtualKeyCode -= 10;
        case 112:       // F1
        case 113:       // F2
        case 114:       // F3
        case 115:       // F4
        case 116:       // F5
        case 117:       // F6
        case 118:       // F7
        case 119:       // F8
        case 120:       // F9
//        case 122:       // F11
        // I don't know why, but F12 is not returned as a key event???
            wKeyEvent = SPEC | (ir.Event.KeyEvent.wVirtualKeyCode - 111 + '0');
            break;

        case 33:    // PgUp
            wKeyEvent = SPEC | 'Z';
            break;

        case 34:    // PgDn
            wKeyEvent = SPEC | 'V';
            break;

        case 35:    // End
            wKeyEvent = SPEC | '>';
            break;

        case 36:    // Home
            wKeyEvent = SPEC | '<';
            break;

        case 37:    // Left
            wKeyEvent = SPEC | 'B';
            break;

        case 38:    // Up
            wKeyEvent = SPEC | 'P';
            break;

        case 39:    // Right
            wKeyEvent = SPEC | 'F';
            break;

        case 40:    // Down
            wKeyEvent = SPEC | 'N';
            break;

        case 45:    // Insert
            wKeyEvent = SPEC | 'C';
            break;

        case 46:    // Delete
            wKeyEvent = SPEC | 'D';
            break;

        case 12:    // Key pad 5
            wKeyEvent = SPEC | 'L';
            break;

        default:
            wKeyEvent = ir.Event.KeyEvent.uChar.AsciiChar;
            break;
    }
    if (ir.Event.KeyEvent.dwControlKeyState & (RIGHT_ALT_PRESSED | LEFT_ALT_PRESSED))
    {
        if (ir.Event.KeyEvent.uChar.AsciiChar >= 'a' &&
            ir.Event.KeyEvent.uChar.AsciiChar <= 'z')
            wKeyEvent = ir.Event.KeyEvent.wVirtualKeyCode;
        wKeyEvent |= ALTD;
    }
    if (ir.Event.KeyEvent.dwControlKeyState & (RIGHT_CTRL_PRESSED | LEFT_CTRL_PRESSED))
        wKeyEvent |= CTRL;
    if (wKeyEvent & (ALTD | SPEC))
        if (ir.Event.KeyEvent.dwControlKeyState & (SHIFT_PRESSED))
            wKeyEvent |= SHFT;

    if (ir.Event.KeyEvent.uChar.AsciiChar == 9 &&
        ir.Event.KeyEvent.dwControlKeyState & (SHIFT_PRESSED))
        wKeyEvent = SHFT | CTRL | 'I';
    wNumKeyEvent++;
}

static void near DispatchEvent(void)
{
    switch (ir.EventType)
    {
        case KEY_EVENT:
            KeyboardEvent();
            break;

        case MOUSE_EVENT:
            MouseEvent();
            break;

        case WINDOW_BUFFER_SIZE_EVENT:
            WindowSizeEvent();
            break;
    }
}

static void near GetCharacter(void)
{
    long dw;
    BOOL bReadAnother;

    while (wNumKeyEvent == 0)
    {
        ReadConsoleInput(hInput, &ir, 1, &dw);
        DispatchEvent();
    }
}

/*----------------------------------------------------------------------*/
/*      ntgetc()                                                        */
/* Get a character from the keyboard.                                   */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntgetc()
{
    if (wNumKeyEvent == 0)    
        GetCharacter();
    wNumKeyEvent = 0;
    return wKeyEvent;
}

#if     TYPEAH
/*----------------------------------------------------------------------*/
/*      typahead()                                                      */
/* Returns true if a key has been pressed.                              */
/*----------------------------------------------------------------------*/

PASCAL NEAR typahead()

{
    DWORD dwCount = 1;

    /* While we have events waiting, process them and if we get a keystroke
     * then return true.  If we exhaust all events and still don't have a
     * keystroke, return false.
     */
     
    while (wNumKeyEvent == 0 && dwCount != 0)
    {
        PeekConsoleInput(hInput, &ir, 1, &dwCount);

        if (dwCount)
        {
            /* This call will set 'wNumKeyEvent' if we have a keyboard 
             * event and will immediatly process any mouse or resize event.
             */
            ReadConsoleInput(hInput, &ir, 1, &dwCount);
            DispatchEvent();
        }
    }
    return ((wNumKeyEvent == 0) ? FALSE : TRUE);
}
#endif

static WORD near ntAttribute(void)
{
    return (revflag ? (cbcolor | (cfcolor << 4)) : ((cbcolor << 4) | cfcolor));
}


/*----------------------------------------------------------------------*/
/*      ntputc()                                                        */
/* Put a character at the current position in the current colors.       */
/* Note that this does not behave the same as putc() or VioWrtTTy().    */
/* This routine does nothing with returns and linefeeds.  For backspace */
/* it puts a space in the previous column and moves the cursor to the   */
/* previous column.  For all other characters, it will display the      */
/* graphic representation of the character and put the cursor in the    */
/* next column (even if that is off the screen.  In practice this isn't */
/* a problem.                                                           */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntputc(int c)
{
    WORD wScreenPos;

    if (c == '\n' || c == '\r')             /* returns and linefeeds */
        return;

    if (c == '\b')                          /* backspace */
        c = ' ', --ntcol;

    wScreenPos = (ntrow * term.t_ncol) + ntcol++;
    ciScreenBuffer[wScreenPos].Char.AsciiChar = c;
    ciScreenBuffer[wScreenPos].Attributes = ntAttribute();
    ntSetUpdateValues();
}


/*----------------------------------------------------------------------*/
/*      nteeol()                                                        */
/* Erase to end of line.                                                */
/*----------------------------------------------------------------------*/

PASCAL NEAR nteeol()
{
    WORD wNum;
    WORD wScreenPos;
    WORD wAttribute;

    wNum = term.t_ncol - ntcol;
    wScreenPos = ntrow * term.t_ncol + ntcol;
    wAttribute = ntAttribute();
    for (; wNum; wNum--)
    {
        ciScreenBuffer[wScreenPos].Char.AsciiChar = ' ';
        ciScreenBuffer[wScreenPos].Attributes = wAttribute;
        wScreenPos++, ntcol++;
    }
    ntSetUpdateValues();
}


/*----------------------------------------------------------------------*/
/*      nteeop()                                                        */
/* Erase to end of page.                                                */
/*----------------------------------------------------------------------*/

PASCAL NEAR nteeop()
{
    WORD wNum;
    WORD wScreenPos;
    WORD wAttribute;

    wNum = (term.t_ncol - ntcol) + ((term.t_ncol * term.t_nrow) - ntrow);
    wScreenPos = ntrow * term.t_ncol + ntcol;
    wAttribute = ntAttribute();
    for (; wNum; wNum--)
    {
        ciScreenBuffer[wScreenPos].Char.AsciiChar = ' ';
        ciScreenBuffer[wScreenPos].Attributes = wAttribute;
        wScreenPos++, ntcol++;
    }
    ntSetUpdateValues();
}

/*----------------------------------------------------------------------*/
/*      ntrev()                                                 */
/* Change reverse video state.                                          */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntrev(state)

int state;      /* TRUE = reverse, FALSE = normal */

{
    revflag = state;
}

/*----------------------------------------------------------------------*/
/*      ntcres()                                                        */
/* Change the screen resolution.                                        */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntcres(char *res)           /* name of desired video mode   */
{
    return TRUE;
}


/*----------------------------------------------------------------------*/
/*      spal()                                                          */
/* Change pallette settings.  (Does nothing.)                           */
/*----------------------------------------------------------------------*/

PASCAL NEAR spal(char *dummy)
{
    return (TRUE);
}


/*----------------------------------------------------------------------*/
/*      ntbeep()                                                        */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntbeep()
{
//    _beep(1200, 80);
    return (TRUE);
}


/*----------------------------------------------------------------------*/
/*      ntopen()                                                        */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntopen()
{
    CONSOLE_SCREEN_BUFFER_INFO Console;

    /* Wed Jan 22 1992 WaltW - Force ntgetc() to call GetCharacter() the
     * first time.
     */
    wNumKeyEvent = 0;

    /* Wed Jan 22 1992 WaltW - This will allocate a console if started from
     * the windows NT program manager.
     */
    AllocConsole();

    /* Wed Jan 22 1992 WaltW - Save the titlebar of the window so we can
     * restore it when we leave.
     */
    GetConsoleTitle(chConsoleTitle, sizeof(chConsoleTitle));

    /* Wed Jan 22 1992 WaltW - Title to MicroEMACS */
    SetConsoleTitle(PROGNAME);

    /* Wed Jan 22 1992 WaltW - Get our standard handles */
    hInput = GetStdHandle(STD_INPUT_HANDLE);
    hOutput = GetStdHandle(STD_OUTPUT_HANDLE);

    GetConsoleScreenBufferInfo(hOutput, &Console);

    term.t_nrow = Console.dwSize.Y - 1;
    term.t_ncol = Console.dwSize.X;
    ntColMin = ntMin = (WORD)-1;
    ntColMax = ntMax = 0;
    revexist = TRUE;
    revflag = FALSE;
    eolexist = TRUE;
    gfcolor = 15;
    gbcolor = 0;
    cfcolor = 7;
    cbcolor = 0;
    return (TRUE);
}

/*----------------------------------------------------------------------*/
/*      ntclose()                                                       */
/* Restore the original video settings.                                 */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntclose()
{
    SetConsoleTitle(chConsoleTitle);
    FreeConsole();
    return (TRUE);
}

/*----------------------------------------------------------------------*/
/*      ntkopen()                                                       */
/* Open the keyboard.                                                   */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntkopen()
{
    GetConsoleMode(hInput, &ConsoleMode);
    OldConsoleMode = ConsoleMode;
    ConsoleMode &= ~(ENABLE_PROCESSED_INPUT | ENABLE_MOUSE_INPUT |
        ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_WINDOW_INPUT);
    ConsoleMode |= ENABLE_WINDOW_INPUT;
    SetConsoleMode(hInput, ConsoleMode);
    return (TRUE);
}


/*----------------------------------------------------------------------*/
/*      ntkclose()                                                      */
/* Close the keyboard.                                                  */
/*----------------------------------------------------------------------*/

PASCAL NEAR ntkclose()
{
    SetConsoleMode(hInput, OldConsoleMode);
    return (TRUE);
}
#endif
